@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: color.c,v 1.85.2.6 2010/10/11 21:47:03 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: color.c,v 1.98 2011/10/28 21:01:16 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - color.c */
@@ -107,6 +107,7 @@ make_palette()
 
     /* ask for suitable number of colours in the palette */
     i = term->make_palette(NULL);
+    sm_palette.colors = i;
     if (i == 0) {
 	/* terminal with its own mapping (PostScript, for instance)
 	   It will not change palette passed below, but non-NULL has to be
@@ -122,7 +123,6 @@ make_palette()
     }
 
     /* set the number of colours to be used (allocated) */
-    sm_palette.colors = i;
     if (sm_palette.use_maxcolors > 0) {
 	if (sm_palette.colorMode == SMPAL_COLOR_MODE_GRADIENT)
 	    sm_palette.colors = i;	/* EAM Sep 2010 - could this be a constant? */
@@ -211,16 +211,11 @@ void ifilled_quadrangle(gpiPoint* icorners)
     term->filled_polygon(4, icorners);
 
     if (pm3d.hidden3d_tag) {
-
 	int i;
 
-	/* Colour has changed, so we must apply line properties again.
-	 * FIXME: It would be cleaner to apply the general line properties
-	 * outside this loop, and limit ourselves to apply_pm3dcolor().
-	 */
-	static struct lp_style_type lp = DEFAULT_LP_STYLE_TYPE;
-	lp_use_properties(&lp, pm3d.hidden3d_tag);
-	term_apply_lp_properties(&lp);
+	/* It should be sufficient to set only the color, but for some */
+	/* reason this causes the svg terminal to lose the fill type.  */
+	term_apply_lp_properties(&pm3d_border_lp);
 
 	term->move(icorners[0].x, icorners[0].y);
 	for (i = 3; i >= 0; i--) {
@@ -486,8 +481,22 @@ cbtick_callback(
 
     /* draw label */
     if (text) {
-	/* get offset */
+	int just;
 	int offsetx, offsety;
+
+	/* Skip label if we've already written a user-specified one here */
+#	define MINIMUM_SEPARATION 0.001
+	while (userlabels) {
+	    if (fabs((place - userlabels->position) / (CB_AXIS.max - CB_AXIS.min))
+		<= MINIMUM_SEPARATION) {
+		text = NULL;
+		break;
+	    }
+	    userlabels = userlabels->next;
+	}
+#	undef MINIMUM_SEPARATION
+
+	/* get offset */
 	map3d_position_r(&(axis_array[axis].ticdef.offset),
 			 &offsetx, &offsety, "cbtics");
 	/* User-specified different color for the tics text */
@@ -502,16 +511,22 @@ cbtick_callback(
 		    hrotate = axis_array[axis].tic_rotate;
 	    if (len > 0) y3 -= len; /* add outer tics len */
 	    if (y3<0) y3 = 0;
+	    just = hrotate ? LEFT : CENTRE;
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;
 	    write_multiline(x2+offsetx, y3+offsety, text,
-			    (hrotate ? LEFT : CENTRE), CENTRE, hrotate,
+			    just, CENTRE, hrotate,
 			    axis_array[axis].ticdef.font);
 	    if (hrotate)
 		(*term->text_angle)(0);
 	} else {
 	    unsigned int x3 = color_box.bounds.xright + (term->h_char);
 	    if (len > 0) x3 += len; /* add outer tics len */
+	    just = LEFT;
+	    if (axis_array[axis].manual_justify)
+		just = axis_array[axis].label.pos;	    
 	    write_multiline(x3+offsetx, y2+offsety, text,
-			    LEFT, CENTRE, 0.0,
+			    just, CENTRE, 0.0,
 			    axis_array[axis].ticdef.font);
 	}
 	term_apply_lp_properties(&border_lp);	/* border linetype */
