@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: mouse.c,v 1.117.2.6 2011/01/18 21:46:14 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: mouse.c,v 1.133.2.1 2012/01/17 04:41:30 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - mouse.c */
@@ -71,6 +71,7 @@ static char *RCSid() { return RCSid("$Id: mouse.c,v 1.117.2.6 2011/01/18 21:46:1
 #endif
 
 #ifdef OS2
+#include <os2.h>
 #include "os2/pm_msgs.h"
 #endif
 
@@ -257,7 +258,7 @@ static void put_label __PROTO((char *label, double x, double y));
 static void
 alert()
 {
-# ifdef GNUPMDRV
+# ifdef OS2
     DosBeep(444, 111);
 # else
 #  ifdef HAVE_LIBREADLINE
@@ -371,22 +372,22 @@ MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double
 static char *
 xy_format()
 {
-    static char format[0xff];
+    static char format[64];
     format[0] = NUL;
-    strcat(format, mouse_setting.fmt);
-    strcat(format, ", ");
-    strcat(format, mouse_setting.fmt);
+    strncat(format, mouse_setting.fmt, 30);
+    strncat(format, ", ", 2);
+    strncat(format, mouse_setting.fmt, 30);
     return format;
 }
 
 static char *
 zoombox_format()
 {
-    static char format[0xff];
+    static char format[64];
     format[0] = NUL;
-    strcat(format, mouse_setting.fmt);
-    strcat(format, "\r");
-    strcat(format, mouse_setting.fmt);
+    strncat(format, mouse_setting.fmt, 30);
+    strncat(format, "\r", 2);
+    strncat(format, mouse_setting.fmt, 30);
     return format;
 }
 
@@ -424,8 +425,25 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 	}
     } else if (mode == MOUSE_COORDINATES_REAL1) {
 	sprintf(s, xy_format(), x, y);	/* w/o brackets */
-    } else if (mode == MOUSE_COORDINATES_ALT && fmt) {
-	sprintf(s, fmt, x, y);	/* user defined format */
+    } else if (mode == MOUSE_COORDINATES_ALT && (fmt || polar)) {
+	if (polar) {
+	    double phi, r;
+	    double rmin = (R_AXIS.autoscale & AUTOSCALE_MIN) ? 0.0 : R_AXIS.set_min;
+	    phi = atan2(y,x);
+	    if (R_AXIS.log)
+		r = AXIS_UNDO_LOG(POLAR_AXIS, x/cos(phi) + AXIS_DO_LOG(POLAR_AXIS, rmin));
+	    else
+		r = x/cos(phi) + rmin;
+	    if (fmt)
+		sprintf(s, fmt, phi/ang2rad, r);
+	    else {
+		sprintf(s, "polar: ");
+		s += strlen(s);
+		sprintf(s, xy_format(), phi/ang2rad, r);
+	    }
+	} else {
+	    sprintf(s, fmt, x, y);	/* user defined format */
+	}
     } else {
 	sprintf(s, xy_format(), x, y);	/* usual x,y values */
     }
@@ -487,7 +505,7 @@ xDateTimeFormat(double x, char *b, int mode)
 #define MKSTR(sp,x,axis)					\
 do {								\
     if (x >= VERYLARGE)	break;					\
-    if (axis_array[axis].is_timedata) { 			\
+    if (axis_array[axis].datatype == DT_TIMEDATE) {		\
 	char *format = copy_or_invent_formatstring(axis);	\
 	while (strchr(format,'\n'))				\
 	     *(strchr(format,'\n')) = ' ';			\
@@ -622,6 +640,9 @@ apply_zoom(struct t_zoom *z)
 	return;
     }
 
+    /* Now we're committed. Notify the terminal the the next replot is a zoom */
+    (*term->layer)(TERM_LAYER_BEFORE_ZOOM);
+
     flip = (is_splot_map && zoom_now->was_splot_map);
     sprintf(s, "set xr[%.12g:%.12g]; set yr[%.12g:%.12g]",
 	       zoom_now->xmin, zoom_now->xmax, 
@@ -643,6 +664,7 @@ apply_zoom(struct t_zoom *z)
 	for (i=0; i<AXIS_ARRAY_SIZE; i++) {
 	    axis_array_copy[i].label = axis_array[i].label;
 	    axis_array_copy[i].ticdef.def.user = axis_array[i].ticdef.def.user;
+	    axis_array_copy[i].ticdef.font = axis_array[i].ticdef.font;
 	}
 	memcpy(axis_array, axis_array_copy, sizeof(axis_array));
 	s[0] = '\0';	/* FIXME:  Is this better than calling replotrequest()? */
@@ -743,21 +765,18 @@ incr_mousemode(const int amount)
 {
     long int old = mouse_mode;
     mouse_mode += amount;
-    if (MOUSE_COORDINATES_ALT == mouse_mode && !mouse_alt_string) {
+    if (MOUSE_COORDINATES_ALT == mouse_mode && !(mouse_alt_string || polar))
 	mouse_mode += amount;	/* stepping over */
-    }
     if (mouse_mode > MOUSE_COORDINATES_ALT) {
-	mouse_mode = MOUSE_COORDINATES_REAL;
-    } else if (mouse_mode < MOUSE_COORDINATES_REAL) {
+	mouse_mode = MOUSE_COORDINATES_REAL1;
+    } else if (mouse_mode <= MOUSE_COORDINATES_REAL) {
 	mouse_mode = MOUSE_COORDINATES_ALT;
-	if (MOUSE_COORDINATES_ALT == mouse_mode && !mouse_alt_string) {
+	if (!(mouse_alt_string || polar))
 	    mouse_mode--;	/* stepping over */
-	}
     }
     UpdateStatusline();
-    if (display_ipc_commands()) {
+    if (display_ipc_commands())
 	fprintf(stderr, "switched mouse format from %ld to %ld\n", old, mouse_mode);
-    }
 }
 
 static void
@@ -1003,13 +1022,13 @@ builtin_nearest_log(struct gp_event_t *ge)
 	    change_y2 = TRUE;
 	
 	if (change_x1)
-	    do_string(axis_array[FIRST_X_AXIS].log ? "unset log x" : "set log x", FALSE);
+	    do_string(axis_array[FIRST_X_AXIS].log ? "unset log x" : "set log x");
 	if (change_y1)
-	    do_string(axis_array[FIRST_Y_AXIS].log ? "unset log y" : "set log y", FALSE);
+	    do_string(axis_array[FIRST_Y_AXIS].log ? "unset log y" : "set log y");
 	if (change_x2 && !splot_map)
-	    do_string(axis_array[SECOND_X_AXIS].log ? "unset log x2" : "set log x2", FALSE);
+	    do_string(axis_array[SECOND_X_AXIS].log ? "unset log x2" : "set log x2");
 	if (change_y2 && !splot_map)
-	    do_string(axis_array[SECOND_Y_AXIS].log ? "unset log y2" : "set log y2", FALSE);
+	    do_string(axis_array[SECOND_Y_AXIS].log ? "unset log y2" : "set log y2");
 	if (!change_x1 && !change_y1 && splot_map)
 	    do_string_replot( Z_AXIS.log ? "unset log z" : "set log z");
 	
@@ -1091,7 +1110,7 @@ static char *
 builtin_decrement_mousemode(struct gp_event_t *ge)
 {
     if (!ge) {
-	return "`builtin-decrement-mousemode`";
+	return "`builtin-previous-mouse-format`";
     }
     incr_mousemode(-1);
     return (char *) 0;
@@ -1101,7 +1120,7 @@ static char *
 builtin_increment_mousemode(struct gp_event_t *ge)
 {
     if (!ge) {
-	return "`builtin-increment-mousemode`";
+	return "`builtin-next-mouse-format`";
     }
     incr_mousemode(1);
     return (char *) 0;
@@ -1287,7 +1306,7 @@ event_keypress(struct gp_event_t *ge, TBOOLEAN current)
 	bind_install_default_bindings();
     }
 
-    if (modifier_mask & Mod_Shift) {
+    if ((modifier_mask & Mod_Shift) && ((c & 0xff) == 0)) {
 	c = toupper(c);
     }
 
@@ -1331,7 +1350,7 @@ event_keypress(struct gp_event_t *ge, TBOOLEAN current)
 		    /* FIXME - Better to clear MOUSE_[XY] than to set it wrongly. */
 		    /*         This may be worth a separate subroutine.           */
 		    load_mouse_variables(0, 0, FALSE, c);
-		do_string(ptr->command, FALSE);
+		do_string(ptr->command);
 		/* Treat as a current event after we return to x11.trm */
 		ge->type = GE_keypress;
 		break;
@@ -1340,7 +1359,7 @@ event_keypress(struct gp_event_t *ge, TBOOLEAN current)
 		break;
 	    /* Let user defined bindings overwrite the builtin bindings */
 	    } else if ((par2 & 1) == 0 && ptr->command) {
-		do_string(ptr->command, FALSE);
+		do_string(ptr->command);
 		break;
 	    } else if (ptr->builtin) {
 		ptr->builtin(ge);
@@ -1424,12 +1443,12 @@ event_buttonpress(struct gp_event_t *ge)
     MousePosToGraphPosReal(mouse_x, mouse_y, &real_x, &real_y, &real_x2, &real_y2);
 
 
-    if (4 == b &&			/* wheel up */
+    if ((b == 4 || b == 6) && /* 4 - wheel up, 6 - wheel left */
 	(!replot_disabled || refresh_ok)	/* Use refresh if available */
 	&& !(paused_for_mouse & PAUSE_BUTTON3)) {
       double xmin, ymin, x2min, y2min;
       double xmax, ymax, x2max, y2max;
-      if (modifier_mask & Mod_Ctrl) {
+      if ((b == 4) && (modifier_mask & Mod_Ctrl)) {
 	if (modifier_mask & Mod_Shift) {
 	  /* zoom in (X axis only) */
 	  double w1=23./25.;
@@ -1469,7 +1488,7 @@ event_buttonpress(struct gp_event_t *ge)
 	}
       } else {
 #if DO_ZOOM_AND_APPLY_ZOOM_FIXED_TO_HANDLE_Z_AXIS
-	if (modifier_mask & Mod_Alt) {
+	if ((b == 4) && (modifier_mask & Mod_Alt)) {
 	  /* scroll in +Z direction */
 	  xmin = rescale(FIRST_X_AXIS, 1., 0.);
 	  ymin = rescale(FIRST_Y_AXIS, 1., 0.);
@@ -1488,7 +1507,7 @@ event_buttonpress(struct gp_event_t *ge)
 	  }
 	} else 
 #endif
-	  if (modifier_mask & Mod_Shift) {
+	  if (((b == 4) && (modifier_mask & Mod_Shift)) || (b == 6)) {
 	  /* scroll left */
           xmin = rescale(FIRST_X_AXIS, 1.1, -.1);
 	  ymin = rescale(FIRST_Y_AXIS, 1., 0.);
@@ -1520,12 +1539,12 @@ event_buttonpress(struct gp_event_t *ge)
 	  }
 	}
       }
-    } else if (5 == b &&			/* wheel down */
+    } else if (((b == 5) || (b == 7)) && /* 5 - wheel down, 7 - wheel right */
 	       (!replot_disabled || refresh_ok)	/* Use refresh if available */
 	       && !(paused_for_mouse & PAUSE_BUTTON3)) {
       double xmin, ymin, x2min, y2min;
       double xmax, ymax, x2max, y2max;
-      if (modifier_mask & Mod_Ctrl) {
+      if ((b == 5) && (modifier_mask & Mod_Ctrl)) {
 	if (modifier_mask & Mod_Shift) {
 	  /* zoom out (X axis only) */
 	  double w1= 23./21.;
@@ -1562,7 +1581,7 @@ event_buttonpress(struct gp_event_t *ge)
 	  }
 	}
       } else {
-	if (modifier_mask & Mod_Shift) {
+	if (((b == 5) && (modifier_mask & Mod_Shift)) || (b == 7)) {
 	  /* scroll right */
 	  xmin = rescale(FIRST_X_AXIS, .9, .1);
 	  ymin = rescale(FIRST_Y_AXIS, 1., 0.);
@@ -1864,7 +1883,8 @@ event_motion(struct gp_event_t *ge)
 	    } else {
 
 		if (relx > rely) {
-		    surface_scale += (mouse_x - start_x) * 2.0 / term->xmax;
+		    surface_lscale += (mouse_x - start_x) * 2.0 / term->xmax;
+		    surface_scale = exp(surface_lscale);
 		    if (surface_scale < 0)
 			surface_scale = 0;
 		} else {
@@ -2039,7 +2059,7 @@ do_event(struct gp_event_t *ge)
 	break;
     case GE_replot:
 	/* used only by ggi.trm */
-	do_string("replot", FALSE);
+	do_string("replot");
 	break;
     case GE_reset:
 	event_reset(ge);
