@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.66.2.5 2009/03/24 11:19:24 mikulik Exp $"); }
+static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.85 2011/09/04 11:06:19 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - pm3d.c */
@@ -31,6 +31,8 @@ static char *RCSid() { return RCSid("$Id: pm3d.c,v 1.66.2.5 2009/03/24 11:19:24
 
 #include <stdlib.h> /* qsort() */
 
+/* Needed by routine filled_quadrangle() in color.c */
+struct lp_style_type pm3d_border_lp;
 
 /*
   Global options for pm3d algorithm (to be accessed by set / show).
@@ -43,9 +45,6 @@ pm3d_struct pm3d = {
     PM3D_SCANS_AUTOMATIC,	/* scans direction is determined automatically */
     PM3D_CLIP_4IN,		/* clipping: all 4 points in ranges */
     0,				/* no pm3d hidden3d is drawn */
-#if PM3D_HAVE_SOLID
-    0,				/* solid (off by default, that means `transparent') */
-#endif /* PM3D_HAVE_SOLID */
     PM3D_EXPLICIT,		/* implicit */
     PM3D_WHICHCORNER_MEAN,	/* color from which corner(s) */
     1,				/* interpolate along scanline */
@@ -56,7 +55,10 @@ typedef struct {
     double gray;
     double z; /* maximal z value after rotation to graph coordinate system */
     gpdPoint corners[4];
-    gpiPoint icorners[4]; /* also if EXTENDED_COLOR_SPECS is not defined */
+#ifdef EXTENDED_COLOR_SPECS
+    gpiPoint icorners[4];
+#endif
+    t_colorspec *border_color;	/* Only used by depthorder processing */
 } quadrangle;
 
 static int allocated_quadrangles = 0;
@@ -65,13 +67,13 @@ static quadrangle* quadrangles = (quadrangle*)0;
 
 /* Internal prototypes for this module */
 static TBOOLEAN plot_has_palette;
-static TBOOLEAN plot_wants_colorbox;
 static double geomean4 __PROTO((double, double, double, double));
 static double median4 __PROTO((double, double, double, double));
 static void pm3d_plot __PROTO((struct surface_points *, int));
 static void pm3d_option_at_error __PROTO((void));
 static void pm3d_rearrange_part __PROTO((struct iso_curve *, const int, struct iso_curve ***, int *));
 static void filled_color_contour_plot  __PROTO((struct surface_points *, int));
+static TBOOLEAN color_from_rgbvar = FALSE;
 
 /*
  * Utility routines.
@@ -85,14 +87,6 @@ static void filled_color_contour_plot  __PROTO((struct surface_points *, int));
 static double
 geomean4 (double x1, double x2, double x3, double x4)
 {
-#if 0
-    /* return 0 if any of the number is negative */
-    if (x1 <= 0) x1 = 1;
-    if (x2 > 0) x1 *= x2;
-    if (x3 > 0) x1 *= x3;
-    if (x4 > 0) x1 *= x4;
-    return pow(x1, 0.25);
-#else
     /* honor signess, i.e. sign(geomean) = sign(prod(x_i)) */
     int neg = (x1 < 0) + (x2 < 0) + (x3 < 0) + (x4 < 0);
     x1 *= x2 * x3 * x4;
@@ -109,7 +103,6 @@ geomean4 (double x1, double x2, double x3, double x4)
     }
 #endif
     return (neg <= 2) ? x1 : -x1;
-#endif
 }
 
 
@@ -168,7 +161,7 @@ z2cb(double z)
     if (Z_AXIS.log && !CB_AXIS.log) /* log z, linear cb */
 	return exp(z * Z_AXIS.log_base); /* unlog(z) */
     if (!Z_AXIS.log && CB_AXIS.log) /* linear z, log cb */
-	return (log(z) / CB_AXIS.log_base);
+	return (z<=0) ? 0.0 : (log(z) / CB_AXIS.log_base);
     /* both are log */
     if (Z_AXIS.base==CB_AXIS.base) /* can we compare double numbers like that? */
 	return z;
@@ -275,10 +268,8 @@ pm3d_rearrange_part(
 	}
     }
 
-#if 0
-    fprintf(stderr, "(pm3d_rearrange_part) invert       = %d\n", *invert);
-    fprintf(stderr, "(pm3d_rearrange_part) invert_order = %d\n", invert_order);
-#endif
+    FPRINTF((stderr, "(pm3d_rearrange_part) invert       = %d\n", *invert));
+    FPRINTF((stderr, "(pm3d_rearrange_part) invert_order = %d\n", invert_order));
 
     for (scanA = src, scan = len - 1, i = 0; scan >= 0; --scan, i++) {
 	if (pm3d.direction == PM3D_SCANS_AUTOMATIC) {
@@ -372,26 +363,33 @@ void pm3d_depth_queue_flush(void)
 	quadrangle* qp;
 	quadrangle* qe;
 	gpdPoint* gpdPtr;
-	gpiPoint* gpiPtr;
+#ifdef EXTENDED_COLOR_SPECS
+	gpdPoint* gpiPtr;
+	double w = trans_mat[3][3];
+#endif
 	vertex out;
 	double z = 0; /* assignment keeps the compiler happy */
-	double w = trans_mat[3][3];
 	int i;
 
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
 	    gpdPtr = qp->corners;
+#ifdef EXTENDED_COLOR_SPECS
 	    gpiPtr = qp->icorners;
+#endif
 
-	    for (i = 0; i < 4; i++, gpdPtr++, gpiPtr++) {
+	    for (i = 0; i < 4; i++, gpdPtr++) {
 
 		map3d_xyz(gpdPtr->x, gpdPtr->y, gpdPtr->z, &out);
 
 		if (i == 0 || out.z > z)
 		    z = out.z;
 
+#ifdef EXTENDED_COLOR_SPECS
 		gpiPtr->x = (unsigned int) ((out.x * xscaler / w) + xmiddle);
 		gpiPtr->y = (unsigned int) ((out.y * yscaler / w) + ymiddle);
+		gpiPtr++;
+#endif
 	    }
 
 	    qp->z = z; /* maximal z value of all four corners */
@@ -401,8 +399,17 @@ void pm3d_depth_queue_flush(void)
 
 	for (qp = quadrangles, qe = quadrangles + current_quadrangle; qp != qe; qp++) {
 
-	    set_color(qp->gray);
+	    if (color_from_rgbvar)
+		set_rgbcolor(qp->gray);
+	    else
+		set_color(qp->gray);
+	    if (pm3d.hidden3d_tag < 0)
+		pm3d_border_lp.pm3d_color = *(qp->border_color);
+#ifdef EXTENDED_COLOR_SPECS
 	    ifilled_quadrangle(qp->icorners);
+#else
+	    filled_quadrangle(qp->corners);
+#endif
 	}
     }
 
@@ -428,6 +435,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     double avgC, gray;
     double cb1, cb2, cb3, cb4;
     gpdPoint corners[4];
+    int interp_i, interp_j;
 #ifdef EXTENDED_COLOR_SPECS
     gpiPoint icorners[4];
 #endif
@@ -435,10 +443,19 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     /* just a shortcut */
     TBOOLEAN color_from_column = this_plot->pm3d_color_from_column;
+    
+    color_from_rgbvar = (this_plot->lp_properties.pm3d_color.type == TC_RGB
+			&&  this_plot->lp_properties.pm3d_color.value == -1);
 
     if (this_plot == NULL)
 	return;
 
+    /* Apply and save the user-requested line properties */
+    pm3d_border_lp = this_plot->lp_properties;
+    if (pm3d.hidden3d_tag > 0)
+	lp_use_properties(&pm3d_border_lp, pm3d.hidden3d_tag);
+    term_apply_lp_properties(&pm3d_border_lp);
+
     if (at_which_z != PM3D_AT_BASE && at_which_z != PM3D_AT_TOP && at_which_z != PM3D_AT_SURFACE)
 	return;
 
@@ -461,6 +478,50 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 
     pm3d_rearrange_scan_array(this_plot, &scan_array, &scan_array_n, &invert, (struct iso_curve ***) 0, (int *) 0, (int *) 0);
 
+    interp_i = pm3d.interp_i;
+    interp_j = pm3d.interp_j;
+
+    if (interp_i <= 0 || interp_j <= 0) {
+	/* Number of interpolations will be determined from desired number of points.
+	   Search for number of scans and maximal number of points in a scan for points
+	   which will be plotted (INRANGE). Then set interp_i,j so that number of points
+	   will be a bit larger than |interp_i,j|.
+	   If (interp_i,j==0) => set this number of points according to DEFAULT_OPTIMAL_NB_POINTS.
+	   Ideally this should be comparable to the resulution of the output device, which
+	   can hardly by done at this high level instead of the driver level.
+   	*/
+	#define DEFAULT_OPTIMAL_NB_POINTS 200
+	int max_scan_pts = 0;
+	int max_scans = 0;
+	int pts;
+	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
+	    scanA = scan_array[scan];
+	    pointsA = scanA->points;
+	    pts = 0;
+	    for (j=0; j<scanA->p_count; j++)
+		if (pointsA[j].type == INRANGE) pts++;
+	    if (pts > 0) {
+		max_scan_pts = GPMAX(max_scan_pts, pts);
+		max_scans++;
+	    }
+	}
+
+	if (interp_i <= 0) {
+	    ii = (interp_i == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_i;
+	    interp_i = floor(ii / max_scan_pts) + 1;
+	}
+	if (interp_j <= 0) {
+	    ii = (interp_j == 0) ? DEFAULT_OPTIMAL_NB_POINTS : -interp_j;
+	    interp_j = floor(ii / max_scans) + 1;
+	}
+#if 0
+	fprintf(stderr, "pm3d.interp_i=%i\t pm3d.interp_j=%i\n", pm3d.interp_i, pm3d.interp_j);
+	fprintf(stderr, "INRANGE: max_scans=%i  max_scan_pts=%i\n", max_scans, max_scan_pts);
+	fprintf(stderr, "seting interp_i=%i\t interp_j=%i => there will be %i and %i points\n",
+		interp_i, interp_j, interp_i*max_scan_pts, interp_j*max_scans);
+#endif
+    }
+
     if (pm3d.direction == PM3D_DEPTH) {
 
 	for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
@@ -475,8 +536,8 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		allocated_quadrangles += GPMAX(scanA->p_count, scanB->p_count) - 1;
 	    }
 	}
-	allocated_quadrangles *= (pm3d.interp_i > 1) ? pm3d.interp_i : 1;
-	allocated_quadrangles *= (pm3d.interp_j > 1) ? pm3d.interp_j : 1;
+	allocated_quadrangles *= (interp_i > 1) ? interp_i : 1;
+	allocated_quadrangles *= (interp_j > 1) ? interp_j : 1;
 	quadrangles = (quadrangle*)gp_realloc(quadrangles, allocated_quadrangles * sizeof (quadrangle), "pm3d_plot->quadrangles");
 	/* DEBUG: fprintf(stderr, "allocated_quadrangles = %d\n", allocated_quadrangles); */
     }
@@ -508,10 +569,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
      * if bilinear interpolation is enabled, allocate memory for the
      * interpolated points here
      */
-    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
-	bl_point = (gpdPoint **)gp_alloc(sizeof(gpdPoint*) * (pm3d.interp_i+1), "bl-interp along scan");
-	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
-	    bl_point[i1] = (gpdPoint *)gp_alloc(sizeof(gpdPoint) * (pm3d.interp_j+1), "bl-interp between scan");
+    if (interp_i > 1 || interp_j > 1) {
+	bl_point = (gpdPoint **)gp_alloc(sizeof(gpdPoint*) * (interp_i+1), "bl-interp along scan");
+	for (i1 = 0; i1 <= interp_i; i1++)
+	    bl_point[i1] = (gpdPoint *)gp_alloc(sizeof(gpdPoint) * (interp_j+1), "bl-interp between scan");
     }
 
     /*
@@ -524,9 +585,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     for (scan = 0; scan < this_plot->num_iso_read - 1; scan++) {
 	scanA = scan_array[scan];
 	scanB = scan_array[scan + 1];
-#if 0
-	printf("\n#IsoCurveA = scan nb %d has %d points   ScanB has %d points\n", scan, scanA->p_count, scanB->p_count);
-#endif
+
+	FPRINTF((stderr,"\n#IsoCurveA = scan nb %d has %d points   ScanB has %d points\n",
+		scan, scanA->p_count, scanB->p_count));
+
 	pointsA = scanA->points;
 	pointsB = scanB->points;
 	/* if the number of points in both scans is not the same, then the
@@ -588,9 +650,10 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		i1 = ii1;
 		ii1 = itmp;
 	    }
-#if 0
-	    fprintf(stderr,"j=%i:  i=%i i1=%i  [%i]   ii=%i ii1=%i  [%i]\n",j,i,i1,scanA->p_count,ii,ii1,scanB->p_count);
-#endif
+
+	    FPRINTF((stderr,"j=%i:  i=%i i1=%i  [%i]   ii=%i ii1=%i  [%i]\n",
+	    		j,i,i1,scanA->p_count,ii,ii1,scanB->p_count));
+
 	    /* choose the clipping method */
 	    if (pm3d.clip == PM3D_CLIP_4IN) {
 		/* (1) all 4 points of the quadrangle must be in x and y range */
@@ -608,7 +671,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    continue;
 	    }
 
-	    if ((pm3d.interp_i <= 1 && pm3d.interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
+	    if ((interp_i <= 1 && interp_j <= 1) || pm3d.direction == PM3D_DEPTH) {
 #ifdef EXTENDED_COLOR_SPECS
 	      if (!supply_extended_color_specs) {
 #endif
@@ -630,7 +693,20 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    cb4 = z2cb(pointsB[ii1].z);
 		}
 		switch (pm3d.which_corner_color) {
-		    case PM3D_WHICHCORNER_MEAN: avgC = (cb1 + cb2 + cb3 + cb4) * 0.25; break;
+		    default:
+		    case PM3D_WHICHCORNER_MEAN:
+			if (color_from_rgbvar) {
+			    int r = (((int)cb1)&0xff0000)+(((int)cb2)&0xff0000)
+			    	  + (((int)cb3)&0xff0000)+(((int)cb4)&0xff0000);
+			    int g = (((int)cb1)&0xff00)+(((int)cb2)&0xff00)
+			    	  + (((int)cb3)&0xff00)+(((int)cb4)&0xff00);
+			    int b = (((int)cb1)&0xff)+(((int)cb2)&0xff)
+			    	  + (((int)cb3)&0xff)+(((int)cb4)&0xff);
+			    avgC = ((r>>2)&0xff0000) + ((g>>2)&0xff00) + ((b>>2)&0xff);
+			} else {
+    			    avgC = (cb1 + cb2 + cb3 + cb4) * 0.25;
+			}
+			break;
 		    case PM3D_WHICHCORNER_GEOMEAN: avgC = geomean4(cb1, cb2, cb3, cb4); break;
 		    case PM3D_WHICHCORNER_MEDIAN: avgC = median4(cb1, cb2, cb3, cb4); break;
 		    case PM3D_WHICHCORNER_MIN: avgC = minimum4(cb1, cb2, cb3, cb4); break;
@@ -639,21 +715,25 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    case PM3D_WHICHCORNER_C2: avgC = cb2; break;
 		    case PM3D_WHICHCORNER_C3: avgC = cb3; break;
 		    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
-		    default: int_error(NO_CARET, "cannot be here"); avgC = 0;
 		}
-		/* transform z value to gray, i.e. to interval [0,1] */
-		gray = cb2gray(avgC);
-#if 0
+
+		if (color_from_rgbvar) /* we were given an explicit color */
+			gray = avgC;
+		else /* transform z value to gray, i.e. to interval [0,1] */
+			gray = cb2gray(avgC);
+
 		/* print the quadrangle with the given color */
-		printf("averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
-		       avgC,
-		       gray,
-		       pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
-		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y);
-#endif
+		FPRINTF((stderr, "averageColor %g\tgray=%g\tM %g %g L %g %g L %g %g L %g %g\n",
+		       avgC, gray, pointsA[i].x, pointsA[i].y, pointsB[ii].x, pointsB[ii].y,
+		       pointsB[ii1].x, pointsB[ii1].y, pointsA[i1].x, pointsA[i1].y));
+
 		/* set the color */
-		if (pm3d.direction != PM3D_DEPTH)
-		    set_color(gray);
+		if (pm3d.direction != PM3D_DEPTH) {
+		    if (color_from_rgbvar)
+			set_rgbcolor(gray);
+		    else
+			set_color(gray);
+		}
 #ifdef EXTENDED_COLOR_SPECS
 	      }
 #endif
@@ -668,7 +748,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    corners[3].x = pointsA[i1].x;
 	    corners[3].y = pointsA[i1].y;
 
-	    if ( pm3d.interp_i > 1 || pm3d.interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
+	    if (interp_i > 1 || interp_j > 1 || at_which_z == PM3D_AT_SURFACE) {
 		/* always supply the z value if
 		 * EXTENDED_COLOR_SPECS is defined
 		 */
@@ -716,56 +796,56 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 	    } else
     		filled_quadrangle(corners, icorners);
 #else
-	    if (pm3d.interp_i > 1 || pm3d.interp_j > 1) {
+	    if (interp_i > 1 || interp_j > 1) {
 		/* Interpolation is enabled.
 		 * interp_i is the # of points along scan lines
 		 * interp_j is the # of points between scan lines
 		 * Algorithm is to first sample i points along the scan lines
 		 * defined by corners[3],corners[0] and corners[2],corners[1]. */
 		int j1;
-		for (i1 = 0; i1 <= pm3d.interp_i; i1++) {
+		for (i1 = 0; i1 <= interp_i; i1++) {
 		    bl_point[i1][0].x = 
-			((corners[3].x - corners[0].x) / pm3d.interp_i) * i1 + corners[0].x;
-		    bl_point[i1][pm3d.interp_j].x = 
-			((corners[2].x - corners[1].x) / pm3d.interp_i) * i1 + corners[1].x;
+			((corners[3].x - corners[0].x) / interp_i) * i1 + corners[0].x;
+		    bl_point[i1][interp_j].x = 
+			((corners[2].x - corners[1].x) / interp_i) * i1 + corners[1].x;
 		    bl_point[i1][0].y =
-			((corners[3].y - corners[0].y) / pm3d.interp_i) * i1 + corners[0].y;
-		    bl_point[i1][pm3d.interp_j].y =
-			((corners[2].y - corners[1].y) / pm3d.interp_i) * i1 + corners[1].y;
+			((corners[3].y - corners[0].y) / interp_i) * i1 + corners[0].y;
+		    bl_point[i1][interp_j].y =
+			((corners[2].y - corners[1].y) / interp_i) * i1 + corners[1].y;
 		    bl_point[i1][0].z =
-			((corners[3].z - corners[0].z) / pm3d.interp_i) * i1 + corners[0].z;
-		    bl_point[i1][pm3d.interp_j].z =
-			((corners[2].z - corners[1].z) / pm3d.interp_i) * i1 + corners[1].z;
+			((corners[3].z - corners[0].z) / interp_i) * i1 + corners[0].z;
+		    bl_point[i1][interp_j].z =
+			((corners[2].z - corners[1].z) / interp_i) * i1 + corners[1].z;
 		    if (color_from_column) {
 			bl_point[i1][0].c =
-			    ((corners[3].c - corners[0].c) / pm3d.interp_i) * i1 + corners[0].c;
-			bl_point[i1][pm3d.interp_j].c =
-			    ((corners[2].c - corners[1].c) / pm3d.interp_i) * i1 + corners[1].c;
+			    ((corners[3].c - corners[0].c) / interp_i) * i1 + corners[0].c;
+			bl_point[i1][interp_j].c =
+			    ((corners[2].c - corners[1].c) / interp_i) * i1 + corners[1].c;
 		    }
 		    /* Next we sample j points between each of the new points
 		     * created in the previous step (this samples between
 		     * scan lines) in the same manner. */
-		    for (j1 = 1; j1 < pm3d.interp_j; j1++) {
+		    for (j1 = 1; j1 < interp_j; j1++) {
 			bl_point[i1][j1].x =
-			    ((bl_point[i1][pm3d.interp_j].x - bl_point[i1][0].x) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].x - bl_point[i1][0].x) / interp_j) *
 				j1 + bl_point[i1][0].x;
 			bl_point[i1][j1].y =
-			    ((bl_point[i1][pm3d.interp_j].y - bl_point[i1][0].y) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].y - bl_point[i1][0].y) / interp_j) *
 				j1 + bl_point[i1][0].y;
 			bl_point[i1][j1].z =
-			    ((bl_point[i1][pm3d.interp_j].z - bl_point[i1][0].z) / pm3d.interp_j) *
+			    ((bl_point[i1][interp_j].z - bl_point[i1][0].z) / interp_j) *
 				j1 + bl_point[i1][0].z;
 			if (color_from_column)
 			    bl_point[i1][j1].c =
-				((bl_point[i1][pm3d.interp_j].c - bl_point[i1][0].c) / pm3d.interp_j) *
+				((bl_point[i1][interp_j].c - bl_point[i1][0].c) / interp_j) *
 				    j1 + bl_point[i1][0].c;
 		    }
 		}
 		/* Once all points are created, move them into an appropriate
 		 * structure and call set_color on each to retrieve the
 		 * correct color mapping for this new sub-sampled quadrangle. */
-		for (i1 = 0; i1 < pm3d.interp_i; i1++) {
-		    for (j1 = 0; j1 < pm3d.interp_j; j1++) {
+		for (i1 = 0; i1 < interp_i; i1++) {
+		    for (j1 = 0; j1 < interp_j; j1++) {
 			corners[0].x = bl_point[i1][j1].x;
 			corners[0].y = bl_point[i1][j1].y;
 			corners[0].z = bl_point[i1][j1].z;
@@ -778,19 +858,17 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			corners[3].x = bl_point[i1][j1+1].x;
 			corners[3].y = bl_point[i1][j1+1].y;
 			corners[3].z = bl_point[i1][j1+1].z;
- 			if (color_from_column) {
- 			    corners[0].c = bl_point[i1][j1].c;
- 			    corners[1].c = bl_point[i1+1][j1].c;
- 			    corners[2].c = bl_point[i1+1][j1+1].c;
- 			    corners[3].c = bl_point[i1][j1+1].c;
- 			}
-#if 0
-			printf("(%g,%g),(%g,%g),(%g,%g),(%g,%g)\n",
-			    corners[0].x, corners[0].y,
-			    corners[1].x, corners[1].y,
-			    corners[2].x, corners[2].y,
-			    corners[3].x, corners[3].y);
-#endif
+			if (color_from_column) {
+			    corners[0].c = bl_point[i1][j1].c;
+			    corners[1].c = bl_point[i1+1][j1].c;
+			    corners[2].c = bl_point[i1+1][j1+1].c;
+			    corners[3].c = bl_point[i1][j1+1].c;
+			}
+
+			FPRINTF((stderr,"(%g,%g),(%g,%g),(%g,%g),(%g,%g)\n",
+			    corners[0].x, corners[0].y, corners[1].x, corners[1].y,
+			    corners[2].x, corners[2].y, corners[3].x, corners[3].y));
+
 			/* If the colors are given separately, we already loaded them above */
 			if (!color_from_column) {
 			    cb1 = z2cb(corners[0].z);
@@ -815,21 +893,29 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 			    case PM3D_WHICHCORNER_C4: avgC = cb4; break;
 			    default: int_error(NO_CARET, "cannot be here"); avgC = 0;
 			}
-			/* transform z value to gray, i.e. to interval [0,1] */
-			gray = cb2gray(avgC);
+
+			if (color_from_rgbvar) /* we were given an explicit color */
+				gray = avgC;
+			else /* transform z value to gray, i.e. to interval [0,1] */
+				gray = cb2gray(avgC);
+
 			if (pm3d.direction != PM3D_DEPTH) {
-			    set_color(gray);
+			    if (color_from_rgbvar)
+				set_rgbcolor(gray);
+			    else
+				set_color(gray);
 			    filled_quadrangle(corners);
 			} else {
 			    /* copy quadrangle */
 			    quadrangle* qp = quadrangles + current_quadrangle;
 			    memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
 			    qp->gray = gray;
+			    qp->border_color = &this_plot->lp_properties.pm3d_color;
 			    current_quadrangle++;
 			}
 		    }
 		}
-	    } else { /* thus (pm3d.interp_i == 1 && pm3d.interp_j == 1) */
+	    } else { /* thus (interp_i == 1 && interp_j == 1) */
 		if (pm3d.direction != PM3D_DEPTH) {
 		    filled_quadrangle(corners);
 		} else {
@@ -837,6 +923,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
 		    quadrangle* qp = quadrangles + current_quadrangle;
 		    memcpy(qp->corners, corners, 4 * sizeof (gpdPoint));
 		    qp->gray = gray;
+		    qp->border_color = &this_plot->lp_properties.pm3d_color;
 		    current_quadrangle++;
 		}
 	    } /* interpolate between points */
@@ -845,7 +932,7 @@ pm3d_plot(struct surface_points *this_plot, int at_which_z)
     } /* loop over scans */
 
     if (bl_point) {
-	for (i1 = 0; i1 <= pm3d.interp_i; i1++)
+	for (i1 = 0; i1 <= interp_i; i1++)
 	    free(bl_point[i1]);
 	free(bl_point);
     }
@@ -915,9 +1002,6 @@ pm3d_reset()
     pm3d.direction = PM3D_SCANS_AUTOMATIC;
     pm3d.clip = PM3D_CLIP_4IN;
     pm3d.hidden3d_tag = 0;
-#if PM3D_HAVE_SOLID
-    pm3d.solid = 0;
-#endif /* PM3D_HAVE_SOLID */
     pm3d.implicit = PM3D_EXPLICIT;
     pm3d.which_corner_color = PM3D_WHICHCORNER_MEAN;
     pm3d.interp_i = 1;
@@ -936,9 +1020,8 @@ pm3d_draw_one(struct surface_points *plot)
 	/* Draw either at 'where' option of the given surface or at pm3d.where
 	 * global option. */
 
-    if (!where[0]) {
+    if (!where[0])
 	return;
-    }
 
     /* for pm3dCompress.awk */
     if (gppsfile && (pm3d.direction != PM3D_DEPTH))
@@ -1020,10 +1103,8 @@ set_plot_with_palette(int plot_num, int plot_mode)
     struct curve_points *this_2dplot = first_plot;
     int surface = 0;
     struct text_label *this_label = first_label;
-    TBOOLEAN want_palette_but_not_colorbox = FALSE;
 
     plot_has_palette = TRUE;
-    plot_wants_colorbox = TRUE;
     /* Is pm3d switched on globally? */
     if (pm3d.implicit == PM3D_IMPLICIT)
 	return;
@@ -1031,22 +1112,16 @@ set_plot_with_palette(int plot_num, int plot_mode)
     /* Check 2D plots */
     if (plot_mode == MODE_PLOT) {
 	while (this_2dplot) {
-#ifdef WITH_IMAGE
 	    if (this_2dplot->plot_style == IMAGE)
 		return;
-#endif
-	    if (this_2dplot->lp_properties.use_palette) {
-		if (this_2dplot->lp_properties.pm3d_color.type <= TC_RGB)
-		    want_palette_but_not_colorbox = TRUE;
-		    /* don't return yet -- decide later whether showing color box is desirable */
-		else
-		    return;
-	    }
-#ifdef EAM_DATASTRINGS
-	    if (this_2dplot->labels &&
-		this_2dplot->labels->textcolor.type >= TC_CB)
+	    if (this_2dplot->lp_properties.use_palette
+	    &&  this_2dplot->lp_properties.pm3d_color.type > TC_RGB)
+		return;
+	    if (this_2dplot->labels
+	    && (this_2dplot->labels->textcolor.type == TC_CB
+	    ||  this_2dplot->labels->textcolor.type == TC_FRAC
+	    ||  this_2dplot->labels->textcolor.type == TC_Z))
 		return;
-#endif
 	    this_2dplot = this_2dplot->next;
 	}
     }
@@ -1057,24 +1132,20 @@ set_plot_with_palette(int plot_num, int plot_mode)
 	while (surface++ < plot_num) {
 	    if (this_3dplot->plot_style == PM3DSURFACE)
 		return;
-#ifdef WITH_IMAGE
 	    if (this_3dplot->plot_style == IMAGE)
 		return;
-#endif
 	    if (this_3dplot->lp_properties.use_palette) {
 	        int type = this_3dplot->lp_properties.pm3d_color.type;
 		if (type == TC_LT || type == TC_LINESTYLE || type == TC_RGB)
-		    want_palette_but_not_colorbox = TRUE;
-		    /* don't return yet -- decide later whether showing color box is desirable */
+		    /* don't return yet */
+		    ;
 		else
 		    /* TC_DEFAULT: splot x with line|lp|dot palette */
 		    return;
 	    }
-#ifdef EAM_DATASTRINGS
 	    if (this_3dplot->labels &&
 		this_3dplot->labels->textcolor.type >= TC_CB)
 		return;
-#endif
 	    this_3dplot = this_3dplot->next_sp;
 	}
     }
@@ -1097,9 +1168,7 @@ set_plot_with_palette(int plot_num, int plot_mode)
 #undef TC_USES_PALETTE
 
     /* Palette with continuous colors is not used. */
-    if (want_palette_but_not_colorbox == FALSE)
-	plot_has_palette = FALSE; /* otherwise it stays TRUE */
-    plot_wants_colorbox = FALSE;
+    plot_has_palette = FALSE; /* otherwise it stays TRUE */
 }
 
 TBOOLEAN
@@ -1111,6 +1180,6 @@ is_plot_with_palette()
 TBOOLEAN
 is_plot_with_colorbox()
 {
-    return plot_wants_colorbox;
+    return plot_has_palette && (color_box.where != SMCOLOR_BOX_NO);
 }
 
