@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: tables.c,v 1.92.2.4 2010/12/05 00:01:23 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: tables.c,v 1.110 2011/11/10 05:15:58 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - tables.c */
@@ -58,6 +58,7 @@ const struct gen_ftable command_ftbl[] =
     { "ca$ll", call_command },
     { "cd", changedir_command },
     { "cl$ear", clear_command },
+    { "do", do_command },
     { "eval$uate", eval_command },
     { "ex$it", exit_command },
     { "f$it", fit_command },
@@ -84,43 +85,20 @@ const struct gen_ftable command_ftbl[] =
     { "she$ll", do_shell },
     { "sh$ow", show_command },
     { "sp$lot", splot_command },
+    { "st$ats", stats_command },
     { "sy$stem", system_command },
     { "test", test_command },
     { "und$efine", undefine_command },
     { "uns$et", unset_command },
     { "up$date", update_command },
+    { "while", while_command },
+    { "{", begin_clause },
+    { "}", end_clause },
     { ";", null_command },
     /* last key must be NULL */
     { NULL, invalid_command }
 };
 
-/* 'plot' and 'splot' */
-/* HBB 990829: unused, yet? */
-/* Lars 991108: yes, because the 'plot' parser is a real bitch :( */
-/* pm 011129: ...and therefore I'm putting it into #if 0 ... #endif.
- * Anyway, this table can't be used as below because some options
- * belong to the group of data file options and others to the group
- * of plot options
- */
-#if 0
-const struct gen_table plot_tbl[] =
-{
-    { "ax$es", P_AXES },
-    { "ax$is", P_AXES },
-    { "bin$ary", P_BINARY },
-    { "ev$ery", P_EVERY },
-    { "i$ndex", P_INDEX },
-    { "mat$rix", P_MATRIX },
-    { "s$mooth", P_SMOOTH },
-    { "thru$", P_THRU },
-    { "t$itle", P_TITLE },
-    { "not$itle", P_NOTITLE },
-    { "u$sing", P_USING },
-    { "w$ith", P_WITH },
-    { NULL, P_INVALID }
-};
-#endif
-
 /* 'plot ax[ei]s' parameter */
 const struct gen_table plot_axes_tbl[] =
 {
@@ -142,6 +120,7 @@ const struct gen_table plot_smooth_tbl[] =
     { "f$requency", SMOOTH_FREQUENCY },
     { "cum$ulative", SMOOTH_CUMULATIVE },
     { "k$density", SMOOTH_KDENSITY },
+    { "cn$ormal", SMOOTH_CUMULATIVE_NORMALISED },
     { NULL, SMOOTH_NONE }
 };
 
@@ -205,8 +184,10 @@ const struct gen_table set_tbl[] =
     { "k$ey", S_KEY },
     { "keyt$itle", S_KEYTITLE },
     { "la$bel", S_LABEL },
-    { "li$nestyle", S_LINESTYLE },
+    { "lines$tyle", S_LINESTYLE },
+    { "linetype$s", S_LINETYPE },
     { "ls", S_LINESTYLE },
+    { "lt", S_LINETYPE },
     { "loa$dpath", S_LOADPATH },
     { "loc$ale", S_LOCALE },
     { "log$scale", S_LOGSCALE },
@@ -253,6 +234,7 @@ const struct gen_table set_tbl[] =
     { "poi$ntsize", S_POINTSIZE },
     { "pol$ar", S_POLAR },
     { "pr$int", S_PRINT },
+    { "psdir", S_PSDIR },
     { "obj$ect", S_OBJECT },
     { "sa$mples", S_SAMPLES },
     { "si$ze", S_SIZE },
@@ -296,6 +278,8 @@ const struct gen_table set_tbl[] =
     { "noy2ti$cs", S_NOY2TICS },
     { "zti$cs", S_ZTICS },
     { "nozti$cs", S_NOZTICS },
+    { "rti$cs", S_RTICS },
+    { "norti$cs", S_NORTICS },
     { "cbti$cs", S_CBTICS },
     { "nocbti$cs", S_NOCBTICS },
 
@@ -342,6 +326,7 @@ const struct gen_table set_tbl[] =
     { "y2zeroa$xis", S_Y2ZEROAXIS },
     { "zzeroa$xis", S_ZZEROAXIS },
     { "zeroa$xis", S_ZEROAXIS },
+    { "rax$is", S_RAXIS },
 
     { "z$ero", S_ZERO },
     { NULL, S_INVALID }
@@ -413,6 +398,8 @@ const struct gen_table set_key_tbl[] =
     { "maxcol$s", S_KEY_MAXCOLS},
     { "maxcolu$mns", S_KEY_MAXCOLS},
     { "maxrow$s", S_KEY_MAXROWS},
+    { "opaque", S_KEY_FRONT},
+    { "noopaque", S_KEY_NOFRONT},
     { NULL, S_KEY_INVALID }
 };
 
@@ -432,6 +419,7 @@ const struct gen_table set_colorbox_tbl[] =
     { "h$orizontal",	S_COLORBOX_HORIZONTAL },
     { "def$ault",	S_COLORBOX_DEFAULT },
     { "u$ser",		S_COLORBOX_USER },
+    { "at",		S_COLORBOX_USER },
     { "bo$rder",	S_COLORBOX_BORDER },
     { "bd$efault",	S_COLORBOX_BDEFAULT },
     { "nobo$rder",	S_COLORBOX_NOBORDER },
@@ -459,6 +447,7 @@ const struct gen_table set_palette_tbl[] =
     { "ps_allcF",	S_PALETTE_PS_ALLCF },
     { "maxc$olors",	S_PALETTE_MAXCOLORS },
     { "gam$ma",         S_PALETTE_GAMMA },
+    { "cubehelix",      S_PALETTE_CUBEHELIX },
     { NULL, S_PALETTE_INVALID }
 };
 
@@ -649,7 +638,9 @@ const struct gen_table show_style_tbl[] =
     { "incr$ement", SHOW_STYLE_INCREMENT },
     { "hist$ogram", SHOW_STYLE_HISTOGRAM },
     { "circ$le", SHOW_STYLE_CIRCLE },
+    { "ell$ipse", SHOW_STYLE_ELLIPSE },
     { "rect$angle", SHOW_STYLE_RECTANGLE },
+    { "boxplot", SHOW_STYLE_BOXPLOT },
     { NULL, SHOW_STYLE_INVALID }
 };
 
@@ -675,11 +666,13 @@ const struct gen_table plotstyle_tbl[] =
     { "boxer$rorbars", BOXERROR },
     { "boxx$yerrorbars", BOXXYERROR },
     { "st$eps", STEPS },
+    { "fillst$eps", FILLSTEPS },
     { "fs$teps", FSTEPS },
     { "his$teps", HISTEPS },
     { "vec$tors", VECTOR },
     { "fin$ancebars", FINANCEBARS },
     { "can$dlesticks", CANDLESTICKS },
+    { "boxplot", BOXPLOT },
     { "pm$3d", PM3DSURFACE },
     { "labels", LABELPOINTS },
     { "ima$ge", IMAGE },
@@ -687,6 +680,7 @@ const struct gen_table plotstyle_tbl[] =
     { "rgba$lpha", RGBA_IMAGE },
 #ifdef EAM_OBJECTS
     { "cir$cles", CIRCLES },
+    { "ell$ipses", ELLIPSES },
 #endif
     { NULL, -1 }
 };
@@ -699,6 +693,7 @@ const struct gen_table filledcurves_opts_tbl[] =
     { "x2", FILLEDCURVES_X2 },
     { "y2", FILLEDCURVES_Y2 },
     { "xy", FILLEDCURVES_ATXY },
+    { "r", FILLEDCURVES_ATR },
     { "above", FILLEDCURVES_ABOVE },
     { "below", FILLEDCURVES_BELOW },
     { NULL, -1 }
