@@ -225,7 +225,7 @@ static const char *prologue_cp1250_ps[] = {
 "/currency/Aogonek/brokenbar/section/dieresis/copyright/Scedilla\n",
 "/guillemotleft/logicalnot/hyphen/registered/Zdotaccent\n",
 "/degree/plusminus/ogonek/lslash/acute/mu/paragraph/bullet\n",
-"/cedilla/aogonek/scedilla/guillemotright/Ydieresis/hungarumlaut/Lcaron\n",
+"/cedilla/aogonek/scedilla/guillemotright/Lcaron/hungarumlaut/lcaron\n",
 "/zdotaccent/Racute/Aacute/Acircumflex/Abreve/Adieresis/Lacute/Cacute\n",
 "/Ccedilla/Ccaron/Eacute/Eogonek/Edieresis/Ecaron/Iacute/Icircumflex\n",
 "/Dcaron/Eth/Nacute/Ncaron/Oacute/Ocircumflex/Ohungarumlaut/Odieresis\n",
@@ -238,6 +238,62 @@ static const char *prologue_cp1250_ps[] = {
 NULL
 };
 
+static const char *prologue_cp1251_ps[] = {
+"%\n",
+"% encoding for code page cp1251 (8-bit Russian)\n",
+"% version 1.0:  - Mainly letters are mapped. The following positions\n",
+"% (JFi)           are left blank (undefined):\n",
+"%               -- first 32 positions,\n",
+"%               -- frame characters,\n",
+"%               -- a few special characters.\n",
+"\n",
+"/reencodeCP1251 {\n",
+"dup dup findfont dup length dict begin\n",
+"{ 1 index /FID ne { def }{ pop pop } ifelse } forall\n",
+"currentdict /CharStrings known {\n",
+"CharStrings /Idieresis known {\n",
+"/Encoding CP1251Encoding def } if\n",
+"} if\n",
+"currentdict end definefont\n",
+"} def\n",
+"/CP1251Encoding [\n",
+"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n",
+"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n",
+"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n",
+"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n",
+"/space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quotesingle\n",
+"/parenleft /parenright /asterisk /plus /comma /minus /period /slash\n",
+"/zero /one /two /three /four /five /six /seven\n",
+"/eight /nine /colon /semicolon /less /equal /greater /question\n",
+"/at /A /B /C /D /E /F /G\n",
+"/H /I /J /K /L /M /N /O\n",
+"/P /Q /R /S /T /U /V /W\n",
+"/X /Y /Z /bracketleft /backslash /bracketright /asciicircum /underscore\n",
+"/grave /a /b /c /d /e /f /g\n",
+"/h /i /j /k /l /m /n /o\n",
+"/p /q /r /s /t /u /v /w\n",
+"/x /y /z /braceleft /bar /braceright /asciitilde /.notdef\n",
+"/afii10051 /afii10052 /quotesinglbase /afii10100 /quotedblbase /ellipsis /dagger /daggerdbl\n",
+"/Euro /perthousand /afii10058 /guilsinglleft /afii10059 /afii10061 /afii10060 /afii10145\n",
+"/afii10099 /quoteleft /quoteright /quotedblleft /quotedblright /bullet /endash /emdash\n",
+"/.notdef /trademark /afii10106 /guilsinglright /afii10107 /afii10109 /afii10108 /afii10193\n",
+"/space /afii10062 /afii10110 /afii10057 /currency /afii10050 /brokenbar /section\n",
+"/afii10023 /copyright /afii10053 /guillemotleft /logicalnot /hyphen /registered /afii10056\n",
+"/degree /plusminus /afii10055 /afii10103 /afii10098 /mu /paragraph /periodcentered\n",
+"/afii10071 /afii61352 /afii10101 /guillemotright /afii10105 /afii10054 /afii10102 /afii10104\n",
+"/afii10017 /afii10018 /afii10019 /afii10020 /afii10021 /afii10022 /afii10024 /afii10025\n",
+"/afii10026 /afii10027 /afii10028 /afii10029 /afii10030 /afii10031 /afii10032 /afii10033\n",
+"/afii10034 /afii10035 /afii10036 /afii10037 /afii10038 /afii10039 /afii10040 /afii10041\n",
+"/afii10042 /afii10043 /afii10044 /afii10045 /afii10046 /afii10047 /afii10048 /afii10049\n",
+"/afii10065 /afii10066 /afii10067 /afii10068 /afii10069 /afii10070 /afii10072 /afii10073\n",
+"/afii10074 /afii10075 /afii10076 /afii10077 /afii10078 /afii10079 /afii10080 /afii10081\n",
+"/afii10082 /afii10083 /afii10084 /afii10085 /afii10086 /afii10087 /afii10088 /afii10089\n",
+"/afii10090 /afii10091 /afii10092 /afii10093 /afii10094 /afii10095 /afii10096 /afii10097\n",
+"] def\n",
+"\n",
+NULL
+};
+
 static const char *prologue_cp437_ps[] = {
 "%\n",
 "% encoding for code page 437 (original MSDOS)\n",
@@ -482,7 +538,7 @@ NULL
 
 static const char *prologue_prologue_ps[] = {
 "%\n",
-"% Gnuplot Prolog Version 4.4 (January 2010)\n",
+"% Gnuplot Prolog Version 4.6 (November 2011)\n",
 "%\n",
 "%/SuppressPDFMark true def\n",
 "%\n",
@@ -494,6 +550,7 @@ static const char *prologue_prologue_ps[] = {
 "/Z {closepath} bind def\n",
 "/C {setrgbcolor} bind def\n",
 "/f {rlineto fill} bind def\n",
+"/g {setgray} bind def\n",
 "/Gshow {show} def   % May be redefined later in the file to support UTF-8\n",
 "/vpt2 vpt 2 mul def\n",
 "/hpt2 hpt 2 mul def\n",
@@ -515,6 +572,7 @@ static const char *prologue_prologue_ps[] = {
 "\tdup 1 lt {pop 1} if 10 mul /udl exch def} def\n",
 "/PL {stroke userlinewidth setlinewidth\n",
 "\tRounded {1 setlinejoin 1 setlinecap} if} def\n",
+"3.8 setmiterlimit\n",
 "% Default Line colors\n",
 "/LCw {1 1 1} def\n",
 "/LCb {0 0 0} def\n",
